DECLARE SUB FindFirstFile (Maske AS STRING, Attribut AS INTEGER, ErrorCode AS INTEGER)
DECLARE SUB FindNextFile (ErrorCode AS INTEGER)
DECLARE FUNCTION GetAttrFile% ()
DECLARE SUB GetDateFile (Month AS INTEGER, Day AS INTEGER, Year AS INTEGER)
DECLARE FUNCTION GetNameFile$ ()
DECLARE FUNCTION GetSizeFile& ()
DECLARE FUNCTION GetTimeCode# ()
DECLARE SUB GetTimeFile (Hour AS INTEGER, Minute AS INTEGER, Second AS INTEGER)
DECLARE FUNCTION TeilZahl% (Zahl AS INTEGER, VonBit AS INTEGER, BisBit AS INTEGER)
REM $INCLUDE: 'qbx.bi'
REM $INCLUDE: 'datim.bi'

SUB Directory (DirName AS STRING, Maske AS STRING, FileName() AS STRING, Anzahl AS INTEGER)

	DIM Fehler AS INTEGER

	Anzahl = 0: Fehler = 0
	' Attribut = 39 sucht alle Dateien inkl. Hidden + System + Read-Only
	' nheres siehe Anhang E
	FindFirstFile DirName + Maske, 39, Fehler
	DO UNTIL Fehler
		Anzahl = Anzahl + 1
		FileName(Anzahl) = GetNameFile
		FindNextFile Fehler
	LOOP

END SUB

' Achtung: Fr Aufruf von DirectoryRek ist es wichtig, da
' 1. DirName + Maske zusammen eine gltige Dateibezeichnung
'    (wie bei DIR) ergeben
' 2. Anzahl zuvor auf 0 gesetzt wird, da das nicht (wie bei Directory)
'    in DirectoryRek selbst geschehen kann, weil diese Prozedur sich
'    selbst aufruft.
'
SUB DirectoryRek (DirName AS STRING, Maske AS STRING, FileName() AS STRING, Anzahl AS INTEGER)

	' diese Konstante gibt an, wieviele direkte Subdirectories ein
	' Verzeichnis maximal haben kann:
	CONST MaxSubDirectories = 20

	DIM Fehler AS INTEGER
	DIM SubDirectory(MaxSubDirectories) AS STRING
	DIM SubDirectoryZaehler AS INTEGER
	DIM NeuDirName AS STRING

	Fehler = 0

	' Attribut = 39 sucht alle Dateien inkl. Hidden + System + Read-Only
	' und Subdirectories, nheres siehe Anhang E
	FindFirstFile DirName + Maske, 39, Fehler
	DO UNTIL Fehler
		Anzahl = Anzahl + 1
		FileName(Anzahl) = DirName + GetNameFile
		FindNextFile Fehler
	LOOP

	' jetzt Subdirectories suchen:
	Fehler = 0
	SubDirectoryZaehler = 0
	' Attribut = 16 sucht Directories
	FindFirstFile DirName + "*.*", 16, Fehler
	DO UNTIL Fehler
		' zustzliche Abfrage, da auch Dateien mit Attribut > 16 gefunden
		' werden:
		IF GetAttrFile AND 16 THEN
			' es ist ein Subdirectory gefunden worden.
			IF LEFT$(GetNameFile, 1) <> "." AND SubDirectoryZaehler < MaxSubDirectories THEN
				SubDirectoryZaehler = SubDirectoryZaehler + 1
				SubDirectory(SubDirectoryZaehler) = GetNameFile
			END IF
		END IF
		FindNextFile Fehler
	LOOP

	' alle Dateien sind in der Liste. Rekursiv werden die
	' Subdirectories abgearbeitet:

	FOR i% = 1 TO SubDirectoryZaehler
		NeuDirName = DirName + SubDirectory(i%) + "\"
		DirectoryRek NeuDirName, Maske, FileName(), Anzahl
	NEXT

END SUB

SUB FindFirstFile (Maske AS STRING, Attribut AS INTEGER, ErrorCode AS INTEGER)

	' Attribut = 0 sucht alle normalen Dateien; fr weitere Informationen
	' siehe Anhang E

	SHARED DTA AS STRING
	DIM Register AS RegTypeX
	DIM FileName AS STRING

	' Interrupt &H21, Funktion &H1A: DOS die Adresse der DTA (Disk Transfer
	' Area) mitteilen, in der es seine Datei-Informationen ablegen kann:
	DTA = SPACE$(128)
	Register.ax = &H1A * 256
	Register.ds = SSEG(DTA)
	Register.dx = SADD(DTA)
	InterruptX &H21, Register, Register

	' Interrupt &H21, Funktion &H4E: Erste Datei suchen, die pat
	FileName = Maske + CHR$(0)
	Register.ax = &H4E * 256
	Register.cx = Attribut
	Register.ds = SSEG(FileName)
	Register.dx = SADD(FileName)
	InterruptX &H21, Register, Register

	IF Register.flags AND 1 THEN
		' Fehler! (Keine Datei vorhanden oder Directory-Angabe falsch)
		ErrorCode = Register.ax
		EXIT SUB
	END IF
	ErrorCode = 0

END SUB

SUB FindNextFile (ErrorCode AS INTEGER)

	SHARED DTA AS STRING
	DIM Register AS RegTypeX

	' Interrupt &H21, Funktion &H1A: DOS die Adresse der DTA (Disk Transfer
	' Area) mitteilen, in der es seine Datei-Informationen ablegen kann:
	' (Mu dauernd aufgerufen werden, weil BASIC von Zeit zu Zeit den String
	' DTA im Speicher herumschiebt und dann die DOS bekannte Adresse nicht
	' mehr stimmt!)
	Register.ax = &H1A * 256
	Register.ds = SSEG(DTA)
	Register.dx = SADD(DTA)
	InterruptX &H21, Register, Register

	' Interrupt &H21, Funktion &H4F: Nchste passende Datei suchen
	Register.ax = &H4F * 256
	InterruptX &H21, Register, Register

	IF Register.flags AND 1 THEN
		' Fehler! (Keine Datei mehr gefunden)
		ErrorCode = Register.ax
	END IF

END SUB

SUB GetDateFile (Month AS INTEGER, Day AS INTEGER, Year AS INTEGER)

	SHARED DTA AS STRING
	DIM Datum AS INTEGER

	' Datum ist in den Bytes 25-26 der DTA gespeichert
	Datum = CVI(MID$(DTA, 25, 2))

	' Tag in den Bits 1-5
	Day = TeilZahl(Datum, 1, 5)
	' Monat in Bits 6-9
	Month = TeilZahl(Datum, 6, 9)
	' Jahr relativ zu 1980 in Bits 10-16
	Year = 1980 + TeilZahl(Datum, 10, 16)

END SUB

FUNCTION GetNameFile$

	SHARED DTA AS STRING
	GetNameFile$ = MID$(DTA, 31, INSTR(31, DTA, CHR$(0)) - 31)

END FUNCTION

FUNCTION GetSizeFile&

	SHARED DTA AS STRING
	GetSizeFile& = ASC(MID$(DTA, 27)) + 256& * ASC(MID$(DTA, 28)) + 65536 * ASC(MID$(DTA, 29)) + 16777216 * ASC(MID$(DTA, 30))

END FUNCTION

FUNCTION GetTimeCode#

	GetTimeFile h%, m%, s%
	GetDateFile mm%, d%, y%
	GetTimeCode = TimeSerial(h%, m%, s%) + DateSerial(mm%, d%, y%)

END FUNCTION

SUB GetTimeFile (Hour AS INTEGER, Minute AS INTEGER, Second AS INTEGER)

	SHARED DTA AS STRING
	DIM Zeit AS INTEGER

	' Zeitangabe steht auf Bytes 23-24 der DTA
	Zeit = CVI(MID$(DTA, 23, 2))

	' Sekunde ist in den ersten 5 Bit gespeichert,
	' allerdings in 2-Sekunden-Schritten
	Second = TeilZahl(Zeit, 1, 5) * 2
	' Minute in den Bits 6-11
	Minute = TeilZahl(Zeit, 6, 11)
	' Stunde in den Bits 12-16
	Hour = TeilZahl(Zeit, 12, 16)

END SUB

' Funktion TeilZahl ermittelt eine Zahl, die in einem
' bestimmten Bit-Bereich einer INTEGER-Zahl gespeichert
' ist. Gltige Bitnummern sind 1 bis 16. Die kleinste Zahl,
' die zurckgegeben wird, ist 0, die grte ist
' 2 ^ (BisBit - VonBit).
'
FUNCTION TeilZahl% (Zahl AS INTEGER, VonBit AS INTEGER, BisBit AS INTEGER)

	z% = 0
	FOR i = VonBit TO BisBit
		IF Zahl AND (2 ^ (i - 1)) THEN z% = z% + 2 ^ (i - VonBit)
	NEXT
	TeilZahl = z%

END FUNCTION

